/* Jeff DeMaagd 1999
	reads in and manipulates a DXF structure.
*/

//	Value types:
//	  0 - 9		String
//	 10 - 59	Floating Point
//	 60 - 79	Integer
//	210 - 239	Floating Point
//	999		Comment

//		0 - entity type
//		2 - section ID (ENTITIES)
//		5 - entity 'handle' - a 4 character HEX value
//		6 - line type
//		8 - layer name
//		100 - object type (ACAD R13 and up)

// My object types:
// 1 = point
// 2 = line segment
// 3 = 3D face (quad)
// 4 = arc
// 4 = arc

#ifndef _DXF_CLASS_
#define _DXF_CLASS_
#include "../geom/geom.h"
#include <iostream>
#include <iomanip>
#include <vector>
#include <string>

#define DXF_UNDEFINED	0
#define DXF_POINT	1
#define DXF_LINE	2	// a line segment (endpoints defined)
#define DXF_3DFACE	3	// a 3d face
#define DXF_CIRCLE	4	// a circle
#define DXF_ARC		5	// an arc
#define DXF_TEXT	6	// not supported yet
#define DNL "\x0D\x0A"		// DOS (N)ew(L)ine.

struct text_t	{
    point_t p1, p2;				//location & alignment points
    char t1[64];				//actual text
    double	a1,	a2,			//rotation and obliquing angles 
		height, xscale, just, flag;  
};

// *****************************************************************
class data_node {

public:

	// define variables.

	int color;		// DXF item 62
	string layer;		// this will be int eventually.  DO NOT USE
	string linetype;	// this will be int eventually.  DO NOT USE	
	string handle;
	int geomtype;
	point_t	e1, e2;		// endpoints to an object.  used for sorting only.
	union {
		point_t		point;
		line_seg_t	line;
		arc_t		arc;
		circle_t	circle;
		quadp_t		quad;
		text_t		text;
	} geom;

	// ***** initialize the data node to default
	data_node() {
		color = 0;
		//layer = 0;
		//linetype = 0;
		geomtype = 0;
	}	// data_node()

//	data_node operator= (data_node & tempnode)
//	{	return (*tempnode);	}

	friend istream & operator>> (istream & input, data_node & element);
	friend ostream & operator<< (ostream & output, const data_node & element);

};	// ***** data_node 

	

// **********************************************************************
class dxf_record {

public:
	int type;		//integer that tells type of data stored
	string record;
	int integer;
	double doub;
	
	friend istream & operator >> (istream & input,  dxf_record & record);
	friend ostream & operator << (ostream & output, dxf_record & record);
};



// **********************************************************************
class DXF
{

public:

	DXF()	{
		min_p = assign_point(0,0,0);
		max_p = assign_point(0,0,0);
	};

	~DXF()	{
	};

/*	int read()	{
		data_node temp;
		data_list.push_back(temp);
		return 0;
	} */

	// updates the known bounds of the DXF objects in store
int maxmin (point_t & check);

	// pulls out information about the objects as a whole
int stats();

	// ***** sort the file based on sort criteria
int sort();

	// ***** find the distances between successive elements
int travel();

// ***** Simply read in the entire data file.
friend istream & operator>> (istream & in, DXF & dxf_read);



// ***** Simply write out the entire data file.
friend ostream & operator<< (ostream & out, const DXF & dxf_write);



// DXF operator= (const DXF dxf_read);
// bool operator== (DXF & dxf_read); 

	vector< data_node > data_list;
	point_t min_p, max_p;		// range holding values

private:



};



#endif	// _DXF_CLASS_
