//Jeff DeMaagd (c)1999 (L)GPL licenced.
// defines the manipulation operations on DXF

#include "dxf.h"

// *************************************************************************
// pulls out interesting stats in the file. (1999/4/25)

int DXF::stats()	{
        int     objs = 0,       points = 0,     lines = 0,      arcs = 0,
                circles = 0,    texts = 0,      faces = 0,      unknowns = 0;
//        point_t min_p, max_p;   //range finders.
        vector<data_node>::iterator it;

	min_p	= assign_point(0,0,0);	max_p	= assign_point(0,0,0);
      
        cout << setw(8);

        for (it = data_list.begin(); it < data_list.end(); it++)	{
		it->e1	= assign_point(0,0,0);	it->e2	= assign_point(0,0,0);

                objs++;
		if (it->geomtype == DXF_LINE)   {
                        lines++;
			it->e1 = it->geom.line.p1;
			it->e2 = it->geom.line.p2;
                        maxmin (it->geom.line.p1);
                        maxmin (it->geom.line.p2); 
                } else if (it->geomtype == DXF_ARC)     {
                        arcs++; 
//			if (it->geom.arc.v1

//			it->e1.x = (it->geom.arc.r1) * sin (radians(it->geom.arc.a1));
//			it->e1.y = (it->geom.arc.r1) * cos (radians(it->geom.arc.a1));
//			it->e1.z = it->geom.arc.p1.z;
//			it->e2.x = (it->geom.arc.r1) * sin (radians(it->geom.arc.a2));
//			it->e2.y = (it->geom.arc.r1) * cos (radians(it->geom.arc.a2));
//			it->e2.z = it->geom.arc.p1.z;

			it->e1 = arc_point (it->geom.arc, it->geom.arc.a1);
			it->e2 = arc_point (it->geom.arc, it->geom.arc.a2);

			maxmin (it->e1);
			maxmin (it->e2);
//			maxmin (it->geom.arc.p1);
                } else if (it->geomtype == DXF_POINT)   {
                        points++;
                        maxmin (it->geom.point);
                } else if (it->geomtype == DXF_CIRCLE)  {
                        circles++;
                        maxmin (it->geom.circle.p1);
                } else if (it->geomtype == DXF_3DFACE)  {
			faces++;
			if (same_point (it->geom.quad.p3, it->geom.quad.p4)) {
				it->e1 = it->geom.quad.p2;
				it->e2 = it->geom.quad.p3;
			} else {
				it->e1 = it->geom.quad.p3;
				it->e2 = it->geom.quad.p4;
			}
                        maxmin (it->geom.quad.p1);
                        maxmin (it->geom.quad.p2);
                        maxmin (it->geom.quad.p3);
                        maxmin (it->geom.quad.p4);
                } else if (it->geomtype == DXF_TEXT)    {
                        texts++;
			maxmin (it->geom.text.p1);
                } else
                        unknowns++;
        }
          
        cout << "\nPoint   count: "	<< points     << "\n";
        cout << "Line    count: "	<< lines      << "\n";
        cout << "Arc     count: "	<< arcs       << "\n";
        cout << "Circle  count: "	<< circles    << "\n";
        cout << "Text    count: "	<< texts      << "\n";
        cout << "Face    count: "	<< faces      << "\n";
        cout << "Unknown count: "	<< unknowns   << "\n";
        cout << "Total objects: "	<< objs       << "\n";
                
        cout << "\nRanges:\n";  
        cout << " X axis:\t" << min_p.x << " to " << max_p.x << "\n";
        cout << " Y axis:\t" << min_p.y << " to " << max_p.y << "\n";
        cout << " Z axis:\t" << min_p.z << " to " << max_p.z << "\n\n";

        return 0;
}



// ************************************************************************
// used by DXF::stats() to test if point is outside current known bounds,
// then corrects wrong entries. (1999/4/25)

int DXF::maxmin (point_t & check)      {

//	cout << check.x << check.y << check.z << max_p.x << max_p.y << max_p.z;

        if (check.x > max_p.x)		max_p.x = check.x;
        if (check.y > max_p.y)		max_p.y = check.y;
        if (check.z > max_p.z)		max_p.z = check.z;
        if (check.x < min_p.x)		min_p.x = check.x;
        if (check.y < min_p.y)		min_p.y = check.y;
        if (check.z < min_p.z)		min_p.z = check.z;

        return 0;
}

// ************************************************************************
int DXF::travel()	{
	double length, tot_len, temp1, temp2;
        point_t tp1;
	data_node temp_data;
        vector<data_node>::iterator it;

        tot_len = 0;
        tp1 = assign_point(0,0,0);

	for (it = data_list.begin(); it < data_list.end(); it++)	{
		temp1 = manhat_dist (tp1, it->e1);
		temp2 = manhat_dist (tp1, it->e2);
		cout << temp1 << '\t' << temp2 << '\t';
		if (temp1 < temp2)	{
			length = temp1;
			tp1 = it->e1;
		} else {
			length = temp2;
			tp1 = it->e2;
		}
		tot_len += length;
		cout << length << "\n";
//		tp1 = it->e2;
	}

	cout << tot_len << " is the total non-entity travel distance.\n";
	return 0;
}

// ************************************************************************
// sorts out all internal data.  Currently, it only sorts lines and arcs.
// hopefully this will eventually traverse into blocks, polylines and 3D faces

int DXF::sort()      {
	double length, tot_len, best_len, temp1, temp2;
        point_t tp1;
	data_node temp_data;
        vector<data_node>::iterator it, it2, loc, loc2, best_loc;

		// re-initialize all vars for sorting	
        tot_len = 0;
	length = 0;
        tp1 = assign_point(0,0,0);

	loc = data_list.begin();
	loc2 = data_list.end();

	for (it = data_list.begin(); it < data_list.end(); it++)	{
	if (it++ != data_list.end())					{
//		cout << "SO\n";
		best_loc = it;
		temp1 = manhat_dist (tp1, it->e1);
		temp2 = manhat_dist (tp1, it->e2);
		if (temp1 < temp2)
			best_len = temp1;
		else
			best_len = temp2;

					// travel distance already != 0:
		if (best_len > 0)	{	
			for (it2 = it++; it2 < data_list.end(); it2++)	{
//				cout << "SI ";
				temp1 = manhat_dist (tp1, it->e1);
				temp2 = manhat_dist (tp1, it->e2);
				if (temp1 < temp2)
					length = temp1;
				else
					length = temp2;
				if (length < best_len)	{
					best_len = length;
					best_loc = it2;
				}
				tot_len += length;
//				cout << length << "\t";
//				cout << "EI\n";
			}
		}

		cout << "Pre-swap\n";

//		if ((best_loc != it) && (best_loc != data_list.end())) {
		if ((best_loc != data_list.end()) && (it != data_list.end())) {
			temp_data = *best_loc;
			*best_loc = *it;
			*it = temp_data;
		}

		cout << "Post-swap\n";

		if (manhat_dist(tp1, it->e1) < manhat_dist(tp1, it->e1))
			tp1 = it->e1;
		else
			tp1 = it->e2;
//		cout << "EO" << endl;
	}
	}

	return 0;
}

