#ifndef _DXF_LIB_
#define _DXF_LIB_
#include <stdio.h>		//read/write
#include <stdlib.h>		//malloc
#include "../geom/geom.h"	//my vector geometry library
#include "../llist/llist.h"	//Ben Van Hof's Linked List library


// Group code  | Following value  |
// range       |                  |
//-------------|------------------|
//    0 - 9    | String           |
//   10 - 59   | Floating-point   |
//   60 - 79   | Integer          |
//  140 - 147  | Floating-point   |
//  170 - 175  | Integer          |
//  210 - 239  | Floating-point   |
//  999        | Comment (string) |
// 1000 - 1009 | String           |
// 1010 - 1059 | Floating-point   |
// 1060 - 1079 | Integer          |


#define DXF_POINT	1
#define DXF_LINE	2
#define DXF_CIRCLE	3
#define DXF_ARC		4
#define DXF_3DFACE	5

FILE *file_in;
int dxf_number;		//DXF record type
char dxf_record[32];	//DXF record data, string alternate
double dxf_double;	//DXF record data, double alternate
int dxf_int;		//DXF record data, integer alternate



// ***********************************************************************
struct dxf_global_data{
	char	cad_version[64];//$ACADVER	AutoCAD version (sorry, I musn't assume too low
	double	angle_base;	//$ANGBASE	Angle 0 direction
	int	angle_dir;	//$ANGDIR	angle direction (0=CW 1=CCW) 
};



// ***********************************************************************
struct dxf_data_node {
	int color;
	int layer;
	int linetype;	
	char handle[8];		// I'm not sure I'll ever use this.
	int geomtype;
	union {
		struct point	point;
		struct line_seg	line;
		struct arc	arc;
		struct circle	circle;
		struct quad	quad;
	} geom;
	struct dxf_data_node *next;
};



// ***********************************************************************
comp_func(struct dxf_data_node *test1, struct dxf_data_node *test1)	{
	if ((test1->color == test2->color) && (test1-> == test2 ->) &&
		(test1->layer == test2->layer) && 
		(test1->linetype == test2->linetype) &&
		(test1->geomtype == test2->geomtype))

};



// ***********************************************************************
struct dxf_layer {
	char layer_name[32];
	struct dxf_layer *next_layer;
};



// ***********************************************************************
struct dxf_linetype {
	struct dxf_layers *dxf_layer_root;
	struct dxf_data_node *dxf_geom_data;
};



// ***********************************************************************
struct dxf_struct {
	struct dxf_layer *layer_root;
	struct dxf_linetype *linetype_root;
	struct dxf_data_node *geom_root;
};



// ***********************************************************************


	//READ the DXF file
int read_dxf (FILE *file_in, struct dxf_struct *dxf_file);	


	//WRITE the DXF info
int write_dxf (FILE *dxf_out);


	//sort the data in the structure
int sort_dxf(FILE *dxf_out, int sort_criteria);


	//destroy the data structure
int destroy_dxf_struct (FILE *dxf_out);	


	//create a new data node
struct dxf_data_node *new_dxf_datanode(void);


	//reads all related info needed on a data element
int read_dxf_element(struct dxf_data_node *new_dnode);


	// reads in a two line 'record' in a DXF file
int read_dxf_record(void);


	// reads in a two line 'record' in a DXF file, with double-specific
	// alternate.  Use only if next record type is known.
int read_dxf_drecord(void);
 
#endif
